Package: formattable
Title: Create 'Formattable' Data Structures
Version: 0.2.1
Authors@R: c(
    person("Kun", "Ren", role = c("aut", "cre"), email = "ken@renkun.me"),
    person("Kenton", "Russell", role = "aut", email = "kent.russell@timelyportfolio.com")
    )
Description: Provides functions to create formattable vectors and data frames.
    'Formattable' vectors are printed with text formatting, and formattable
    data frames are printed with multiple types of formatting in HTML
    to improve the readability of data presented in tabular form rendered in
    web pages.
Depends: R (> 3.0.2)
Date: 2021-01-05
Imports: methods, htmltools, htmlwidgets, knitr, rmarkdown
License: MIT + file LICENSE
LazyData: true
URL: https://renkun-ken.github.io/formattable/,
        https://github.com/renkun-ken/formattable
BugReports: https://github.com/renkun-ken/formattable/issues
ByteCompile: TRUE
Suggests: testthat, DT, shiny, covr
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2021-01-07 01:43:38 UTC; ken
Author: Kun Ren [aut, cre],
  Kenton Russell [aut]
Maintainer: Kun Ren <ken@renkun.me>
Repository: CRAN
Date/Publication: 2021-01-07 20:20:10 UTC
