# Generated by roxygen2: do not edit by hand

S3method("%%",formattable)
S3method("&",formattable)
S3method("*",formattable)
S3method("+",formattable)
S3method("-",formattable)
S3method("/",formattable)
S3method("[",formattable)
S3method("[<-",formattable)
S3method("[[",formattable)
S3method("[[<-",formattable)
S3method("|",formattable)
S3method(accounting,character)
S3method(accounting,default)
S3method(all,formattable)
S3method(any,formattable)
S3method(as.character,formattable)
S3method(as.datatable,formattable)
S3method(as.htmlwidget,formattable)
S3method(as.list,formattable)
S3method(c,formattable)
S3method(comma,character)
S3method(comma,default)
S3method(csscolor,character)
S3method(csscolor,matrix)
S3method(cummax,formattable)
S3method(cummin,formattable)
S3method(cumsum,formattable)
S3method(currency,character)
S3method(currency,default)
S3method(diff,formattable)
S3method(format,formattable)
S3method(formattable,Date)
S3method(formattable,POSIXct)
S3method(formattable,POSIXlt)
S3method(formattable,data.frame)
S3method(formattable,default)
S3method(formattable,factor)
S3method(formattable,formattable)
S3method(formattable,logical)
S3method(formattable,numeric)
S3method(formattable,table)
S3method(knit_print,formattable)
S3method(max,formattable)
S3method(mean,formattable)
S3method(median,formattable)
S3method(min,formattable)
S3method(percent,character)
S3method(percent,default)
S3method(print,formattable)
S3method(print,formatter)
S3method(quantile,formattable)
S3method(rep,formattable)
S3method(sum,formattable)
S3method(unique,formattable)
export(accounting)
export(area)
export(as.datatable)
export(as.htmlwidget)
export(color_bar)
export(color_text)
export(color_tile)
export(comma)
export(csscolor)
export(currency)
export(digits)
export(format_table)
export(formattable)
export(formattableOutput)
export(formatter)
export(gradient)
export(icontext)
export(is.formattable)
export(normalize)
export(normalize_bar)
export(percent)
export(prefix)
export(proportion)
export(proportion_bar)
export(qrank)
export(renderFormattable)
export(scientific)
export(style)
export(suffix)
export(vmap)
importFrom(grDevices,col2rgb)
importFrom(htmltools,attachDependencies)
importFrom(htmltools,doRenderTags)
importFrom(htmltools,tag)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(htmlwidgets,createWidget)
importFrom(htmlwidgets,shinyRenderWidget)
importFrom(htmlwidgets,shinyWidgetOutput)
importFrom(knitr,asis_output)
importFrom(knitr,kable)
importFrom(knitr,knit_print)
importFrom(rmarkdown,html_dependency_bootstrap)
importFrom(rmarkdown,html_dependency_jquery)
importFrom(stats,median)
importFrom(stats,quantile)
